-----------------------------------------------------------------------------
-- FuBar stuff
-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("Aloft")
local SML = AceLibrary("SharedMedia-1.0")

Aloft = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "FuBarPlugin-2.0")

Aloft:RegisterDB("AloftDB")
Aloft:RegisterDefaults('profile', 
	{
		nameplateTextSize = 10,
		healthBarTexture = "Blizzard",
		raidIconVisible = true,
		raidIconPlacement = {
								point			= "RIGHT",
								relativeToPoint	= "LEFT",
								offsetX			= 0,
								offsetY			= 0,
								size			= 20
							},
		eliteIconVisible = true,
		eliteIconPlacement = {
								point			= "RIGHT",
								relativeToPoint	= "LEFT",
								offsetX			= 0,
								offsetY			= 0,
								size			= 16
							},
	}
)

Aloft.version = "1.0"
Aloft.hasIcon = "Interface\\Icons\\Spell_Frost_Wisp"	-- Placeholder
Aloft.defaultPosition = 'LEFT'
Aloft.hideWithoutStandby = true
Aloft.independentProfile = true

-----------------------------------------------------------------------------

local numberOfWorldChildren = 0
local nameplateList = { }

-----------------------------------------------------------------------------

function Aloft:OnInitialize()
	self:SetText(L["Aloft"])
end

function Aloft:OnEnable()
	self:ScheduleRepeatingEvent("AloftUpdate", self.Update, 0.2, self)
end

function Aloft:OnDisable()
end

function Aloft:IsNameplateFrame(frame)
	if frame:GetName() then
		return false
	end

	local borderFrame = select(1, frame:GetRegions())	
	if not borderFrame or borderFrame:GetObjectType() ~= "Texture" or borderFrame:GetTexture() ~= "Interface\\Tooltips\\Nameplate-Border" then
		return false
	end
	
	return true
end

function Aloft:ProcessChildFrames(...)
	for i = 1, select('#', ...) do
		local frame = select(i, ...)
		if self:IsNameplateFrame(frame) then 
			if not nameplateList[frame] then
				self:SetupNamePlate(frame)
				nameplateList[frame] = frame
			end
		end
	end
end

function Aloft:Update()
	local newNumberOfWorldChildren = WorldFrame:GetNumChildren()
	if newNumberOfWorldChildren ~= numberOfWorldChildren then
		numberOfWorldChildren = newNumberOfWorldChildren

		self:ProcessChildFrames(WorldFrame:GetChildren())
	end
end

if Prat_PlayerNames then
	Aloft.PratPlayerNamesUpdateMouseOver = Prat_PlayerNames.updateMouseOver
	Prat_PlayerNames.updateMouseOver = function()
		if UnitIsPlayer("mouseover") and UnitIsFriend("player", "mouseover") and not Prat_PlayerNames.Classes[UnitName("mouseover")] then
			Aloft.PratPlayerNamesUpdateMouseOver(Prat_PlayerNames)
			Aloft:UpdateHealthBarColourForUnitNamed(UnitName("mouseover"))
		end
	end

end

function Aloft:UpdateHealthBarColourForUnitNamed(name)
	for frame in pairs(nameplateList) do	
		if frame.aloftData.name == name then
			self:UpdateNameplate(frame)
		end
	end
end

-----------------------------------------------------------------------------

function Aloft:PlaceFrame(frame, healthBar, visible, placement)
	if visible then
		frame:ClearAllPoints()
		frame:SetPoint(placement.point, healthBar, placement.relativeToPoint, placement.offsetX, placement.offsetY)
		frame:SetWidth(placement.size)
		frame:SetHeight(placement.size)
		frame:SetAlpha(1)
	else
		frame:SetAlpha(0)
	end
end

-----------------------------------------------------------------------------

function Aloft:UpdateHealthBarColor(frame, healthBar)
	local borderFrame, frame2, frame3, highlightFrame, nameFrame, levelTextFrame, eliteIconFrame, raidIconFrame = frame:GetRegions()

--	min,max = healthBar:GetMinMaxValues()
--	ChatFrame3:AddMessage("HB: "..frame.aloftData.name.." - "..min.."/"..max)

	if Prat_PlayerNames and frame.aloftData.type == "friendlyPlayer" and Prat_PlayerNames.Classes[frame.aloftData.name] then
		local color = RAID_CLASS_COLORS[string.upper(Prat_PlayerNames.Classes[frame.aloftData.name])]
		if color then
			healthBar:SetStatusBarColor( color.r, color.g, color.b, 0.75);
			return
		end
	end
--	ChatFrame3:AddMessage("Unable to set color for: "..frame.aloftData.name.."."..nameFrame:GetText().."."..frame.aloftData.type)
	
	healthBar:SetAlpha(0.75)
end

function Aloft:UpdateNameplate(frame)
	local borderFrame, frame2, frame3, highlightFrame, nameFrame, levelTextFrame, eliteIconFrame, raidIconFrame = frame:GetRegions()
	local healthBar = frame:GetChildren()

	self:UpdateHealthBarColor(frame, healthBar)
end

function Aloft:OnHealthBarValueChanged(frame)
	local borderFrame, frame2, frame3, highlightFrame, nameFrame, levelTextFrame, eliteIconFrame, raidIconFrame = frame:GetRegions()
	local healthBar = frame:GetChildren()
	
	if frame.aloftData.name ~= nameFrame:GetText() then
		self:OnNameplateShow(frame)
	end
	self:UpdateHealthBarColor(frame, healthBar)
end

function Aloft:OnNameplateShow(frame)
	-- When a nameplate is first shown, grab some data about it
	
	local borderFrame, frame2, frame3, highlightFrame, nameFrame, levelTextFrame, eliteIconFrame, raidIconFrame = frame:GetRegions()
	local healthBar = frame:GetChildren()

	frame.aloftData.name = nameFrame:GetText()
	
	local r, g, b = healthBar:GetStatusBarColor();
	r, g, b = floor(r * 256), floor(g*256), floor(b*256)

--	ChatFrame3:AddMessage("rgb = "..r.."."..g.."."..b)
	if r == 255 and g == 0 and b == 0 then
		frame.aloftData.type = "hostile"
	elseif r == 0 and g == 0 and b == 255 then
		frame.aloftData.type = "friendlyPlayer"
	elseif r == 0 and g == 255 and b == 0 then
		frame.aloftData.type = "friendlyNPC"
	elseif r == 255 and g == 255 and b == 0 then
		frame.aloftData.type = "neutral"
	end

	levelTextFrame:Hide()
	eliteIconFrame:Hide()

	nameFrame:SetAlpha(0.75)
	
	-- The frame has an active region for detecting hits that is given roughly by this formula.
	frame:SetHeight(healthBar:GetHeight()*1.6)
	frame:SetWidth(healthBar:GetWidth()*1.05)
	
	self:UpdateNameplate(frame)
end

function Aloft:OnNameplateHide(frame)
	frame.aloftData.name = nil
	frame.aloftData.type = "unknown"
end

-----------------------------------------------------------------------------

local function OnNameplateShow()
	Aloft:OnNameplateShow(this)
end

local function OnNameplateHide()
	Aloft:OnNameplateHide(this)
end

local function OnHealthBarValueChanged()
	Aloft:OnHealthBarValueChanged(this:GetParent())
end

-----------------------------------------------------------------------------

function Aloft:SetupNamePlate(frame)
	frame.aloftData = { type = "unknown" }

	local borderFrame, frame2, frame3, highlightFrame, nameFrame, levelTextFrame, eliteIconFrame, raidIconFrame = frame:GetRegions()
	local healthBar = frame:GetChildren()

	borderFrame:Hide()
	
	-- Using SetAlpha permanently hides the highlight feature
	highlightFrame:SetAlpha(0)	
	
	nameFrame:SetAllPoints(healthBar)
	nameFrame:SetFont(nameFrame:GetFont(), self.db.profile.nameplateTextSize, "OUTLINE")
	nameFrame:SetJustifyH("LEFT")

	self:PlaceFrame(raidIconFrame, healthBar, self.db.profile.raidIconVisible, self.db.profile.raidIconPlacement)
	self:PlaceFrame(eliteIconFrame, healthBar, self.db.profile.eliteIconVisible, self.db.profile.eliteIconPlacement)

	frame:SetScript("OnShow", OnNameplateShow)
	frame:SetScript("OnHide", OnNameplateHide)
	healthBar:SetScript("OnValueChanged", OnHealthBarValueChanged)
	healthBar:SetStatusBarTexture(SML:Fetch("statusbar", self.db.profile.healthBarTexture))
	
	self:OnNameplateShow(frame)
end

-----------------------------------------------------------------------------

local SML_font = SML:List("font")
local SML_statusbar = SML:List("statusbar")

Aloft.OnMenuRequest = 
{
	handler = Aloft,
	type = 'group',
	args = 
	{
		nameplateTextSize =
		{
			type = 'range',
			name = L["Nameplate text size"],
			desc = L["Sets the font height of the nameplate text"],
			max = 20,
			min = 5,
			step = 1,
			get = function() return Aloft.db.profile.nameplateTextSize end,
			set = function(value) Aloft.db.profile.nameplateTextSize = value Aloft:UpdateNameplateTextSize() end
		},
		healthBarTexture =
		{
			type = "text", 
			name = L["Health bar texture"], 
			desc = L["Sets the health bar texture"],
			get = function() return Aloft.db.profile.healthBarTexture end,
			set = function(v) Aloft.db.profile.healthBarTexture = v Aloft:UpdateHealthBarTextures() end,
			validate = SML_statusbar,
		},
		raidIcon =
		{
			type = 'group',
			name = L["Raid Icon"],
			desc = L["Raid icon placement options"],
			args =
			{
				visible = 
				{
					type = 'toggle',
					name = L["Enable"],
					desc = L["Enable raid icon display on the nameplate"],
					get = function() return Aloft.db.profile.raidIconVisible end,
					set = function(v) Aloft.db.profile.raidIconVisible = v Aloft:UpdateRaidIcons() end
				},
				anchor = {
					type = "text", 
					name = L["Anchor"], 
					desc = L["Sets the anchor for the raid icon"],
					get = function() return Aloft.db.profile.raidIconPlacement.point end,
					set = function(v) Aloft.db.profile.raidIconPlacement.point = v Aloft:UpdateRaidIcons() end,
					validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
				},
				anchorto = {
					type = "text", 
					name = L["Anchor To"], 
					desc = L["Sets the relative point on the health bar to anchor the raid icon"],
					get = function() return Aloft.db.profile.raidIconPlacement.relativeToPoint end,
					set = function(v) Aloft.db.profile.raidIconPlacement.relativeToPoint = v Aloft:UpdateRaidIcons() end,
					validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
				},
				size =
				{
					type = 'range',
					name = L["Size"],
					desc = L["Size in pixels of the raid icon"],
					min = 4,
					max = 32,
					step = 1,
					get = function() return Aloft.db.profile.raidIconPlacement.size end,
					set = function(v) Aloft.db.profile.raidIconPlacement.size = v Aloft:UpdateRaidIcons() end
				},
				offsetX =
				{
					type = 'range',
					name = L["X Offset"],
					desc = L["X offset of the raid icon"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return Aloft.db.profile.raidIconPlacement.offsetX end,
					set = function(v) Aloft.db.profile.raidIconPlacement.offsetX = v Aloft:UpdateRaidIcons() end
				},
				offsetY =
				{
					type = 'range',
					name = L["Y Offset"],
					desc = L["Y offset of the raid icon"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return Aloft.db.profile.raidIconPlacement.offsetY end,
					set = function(v) Aloft.db.profile.raidIconPlacement.offsetY = v Aloft:UpdateRaidIcons() end
				},
			},
		},
		eliteIcon =
		{
			type = 'group',
			name = L["Elite Icon"],
			desc = L["Elite icon placement options"],
			args =
			{
				visible = 
				{
					type = 'toggle',
					name = L["Enable"],
					desc = L["Enable elite icon display on the nameplate"],
					get = function() return Aloft.db.profile.eliteIconVisible end,
					set = function(v) Aloft.db.profile.eliteIconVisible = v Aloft:UpdateEliteIcons() end
				},
				anchor = {
					type = "text", 
					name = L["Anchor"], 
					desc = L["Sets the anchor for the elite icon"],
					get = function() return Aloft.db.profile.eliteIconPlacement.point end,
					set = function(v) Aloft.db.profile.eliteIconPlacement.point = v Aloft:UpdateRaidIcons() end,
					validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
				},
				anchorto = {
					type = "text", 
					name = L["Anchor To"], 
					desc = L["Sets the relative point on the health bar to anchor the elite icon"],
					get = function() return Aloft.db.profile.eliteIconPlacement.relativeToPoint end,
					set = function(v) Aloft.db.profile.eliteIconPlacement.relativeToPoint = v Aloft:UpdateRaidIcons() end,
					validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
				},
				size =
				{
					type = 'range',
					name = L["Size"],
					desc = L["Size in pixels of the elite icon"],
					min = 4,
					max = 32,
					step = 1,
					get = function() return Aloft.db.profile.eliteIconPlacement.size end,
					set = function(v) Aloft.db.profile.eliteIconPlacement.size = v Aloft:UpdateEliteIcons() end
				},
				offsetX =
				{
					type = 'range',
					name = L["X Offset"],
					desc = L["X offset of the elite icon"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return Aloft.db.profile.eliteIconPlacement.offsetX end,
					set = function(v) Aloft.db.profile.eliteIconPlacement.offsetX = v Aloft:UpdateEliteIcons() end
				},
				offsetY =
				{
					type = 'range',
					name = L["Y Offset"],
					desc = L["Y offset of the elite icon"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return Aloft.db.profile.eliteIconPlacement.offsetY end,
					set = function(v) Aloft.db.profile.eliteIconPlacement.offsetY = v Aloft:UpdateEliteIcons() end
				},
			},
		},
	},
}

Aloft:RegisterChatCommand({"/aloft"}, Aloft.OnMenuRequest)

function Aloft:UpdateNameplateTextSize()
	for frame in pairs(nameplateList) do	
		local borderFrame, frame2, frame3, highlightFrame, nameFrame, levelTextFrame, eliteIconFrame, raidIconFrame = frame:GetRegions()
		nameFrame:SetFont(nameFrame:GetFont(), self.db.profile.nameplateTextSize, "OUTLINE")
	end
end

function Aloft:UpdateHealthBarTextures()
	for frame in pairs(nameplateList) do	
		local healthBar = frame:GetChildren()
		healthBar:SetStatusBarTexture(SML:Fetch("statusbar", self.db.profile.healthBarTexture))
	end
end

function Aloft:UpdateRaidIcons()
	for frame in pairs(nameplateList) do	
		local borderFrame, frame2, frame3, highlightFrame, nameFrame, levelTextFrame, eliteIconFrame, raidIconFrame = frame:GetRegions()
		local healthBar = frame:GetChildren()
		Aloft:PlaceFrame(raidIconFrame, healthBar, self.db.profile.raidIconVisible, self.db.profile.raidIconPlacement)
	end
end

function Aloft:UpdateEliteIcons()
	for frame in pairs(nameplateList) do	
		local borderFrame, frame2, frame3, highlightFrame, nameFrame, levelTextFrame, eliteIconFrame, raidIconFrame = frame:GetRegions()
		local healthBar = frame:GetChildren()
		Aloft:PlaceFrame(eliteIconFrame, healthBar, self.db.profile.eliteIconVisible, self.db.profile.eliteIconPlacement)
	end
end

-----------------------------------------------------------------------------
